/*
 * ResponseParser.js
 * Yipeng Huang yh2315@columbia.edu
 * Columbia University
 * Module that provides XML parsing methods.
 */

/*
 * Parse the XML document contained in the string argument and return
 * a Document object that represents it.
 */
function parser(lostResponseString) {
    if (typeof DOMParser != "undefined") {
        // Mozilla, Firefox, and related browsers		
        return (new DOMParser()).parseFromString(lostResponseString, "application/xml");
    }
    else if (typeof ActiveXObject != "undefined") {
        // Internet Explorer.
        var doc = XML.newDocument();
        // Create an empty document
        doc.loadXML(lostResponseString);
        //  Parse text into it
        return doc;
        // Return it
    }
    else {
        // As a last resort, try loading the document from a data: URL
        // This is supposed to work in Safari. Thanks to Manos Batsis and
        // his Sarissa library (sarissa.sourceforge.net) for this technique.
        var url = "data:text/xml;charset=utf-8," + encodeURIComponent(lostResponseString);
        var request = new XMLHttpRequest();
        request.open("GET", url, false);
        request.send(null);
        return request.responseXML;
    }
}

/*
 * Parse mapping objects.
 */
function parseMappings(lostResponseString) {

    // parsing is done using both dom and json tools
    var domParser = parser(lostResponseString);
    var jsonParser = xml2json.parser(lostResponseString);

    // initiate the array that will hold the mapping opbjects
    var mappingElements = domParser.getElementsByTagName("mapping");
    var mappings = new Array(mappingElements.length);

    // document.write(xml2json.parser(lostResponseString,'',"html"));
    for (var i = 0; i < mappingElements.length; i++) {

        var mappingElement = mappingElements[i];
        mapping = {};

        mapping.expires = mappingElement.getAttribute('expires');
        mapping.lastUpdated = mappingElement.getAttribute('lastUpdated');
        mapping.source = mappingElement.getAttribute('source');
        mapping.sourceId = mappingElement.getAttribute('sourceId');
        mapping.displayName = mappingElement.getElementsByTagName('displayName')[0].childNodes[0].nodeValue;
        mapping.service = mappingElement.getElementsByTagName('service')[0].childNodes[0].nodeValue;

        // there may be multiple uris
        mapping.uri = new Array(mappingElement.getElementsByTagName('uri').length);
        for (var j = 0; j < mappingElement.getElementsByTagName('uri').length; j++) {
            mapping.uri[j] = mappingElement.getElementsByTagName('uri')[j].childNodes[0].nodeValue;
        }

        // there may be multiple service numbers
        mapping.serviceNumber = new Array(mappingElement.getElementsByTagName('serviceNumber').length);
        for (var j = 0; j < mappingElement.getElementsByTagName('serviceNumber').length; j++) {
            mapping.serviceNumber[j] = mappingElement.getElementsByTagName('serviceNumber')[j].childNodes[0].nodeValue;
        }

        // service boundary is done using json parsing
        // if reference
        if (typeof jsonParser.findserviceresponse.mapping.serviceboundaryreference !== "undefined") {
            mapping.serviceBoundary = jsonParser.findserviceresponse.mapping.serviceboundaryreference;
        }
        // if civic
        else if (typeof jsonParser.findserviceresponse.mapping.serviceboundary.civicaddress !== "undefined") {
            mapping.serviceBoundary = jsonParser.findserviceresponse.mapping.serviceboundary.civicaddress;
        }
        // if polygon
        else if (typeof jsonParser.findserviceresponse.mapping.serviceboundary.p2_polygon) {
            mapping.serviceBoundary = jsonParser.findserviceresponse.mapping.serviceboundary.p2_polygon.p2_exterior.p2_linearring.p2_pos;
        }
        // mapping.serviceBoundary = mappingElement.getElementsByTagName('serviceBoundary')[0];

        // LOST EXTENSION
        // service location
        if (mappingElement.getElementsByTagName('serviceLocation')[0]) {
            mapping.serviceLocation = mappingElement.getElementsByTagName('serviceLocation')[0].childNodes[0].childNodes[0].nodeValue;
        }

        // service location
        mapping.civicAddress = jsonParser.findserviceresponse.mapping.civicaddress;

        mappings[i] = mapping;
    }
    return mappings;
}


/*
 * Parse warnings, if there are any.
 */
function parseWarnings(lostResponseString) {

    // parsing is done using both dom and json tools
    var domParser = parser(lostResponseString);
    var jsonParser = xml2json.parser(lostResponseString);

    return domParser.getElementsByTagName("warnings");

}

/*
 * Parse location validation information.
 */
function parseLocationValidation(lostResponseString) {

    // parsing is done using both dom and json tools
    var domParser = parser(lostResponseString);
    var jsonParser = xml2json.parser(lostResponseString);

    // var locationValidation = mappingElement.getElementsByTagName('locationValidation')[0];
    return jsonParser.findserviceresponse.locationvalidation;

}

// function parseRedirect(lostResponseString) {
//     }